/*
 * =============================================================================
 *
 * D I S T A N C E   T R A N S F O R M   R O U T I N E S  --  HEADER
 *
 * $Revision: 1.5 $ $Date: 2003/02/26 15:26:10 $
 *
 * ANIMAL - ANIMAL IMage Processing LibrarY
 * Copyright (C) 2002,2003  Ricardo Fabbri <rfabbri@if.sc.usp.br>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * =============================================================================
 */ 
#ifndef DT_H
#define DT_H 1

#include <animal/common.h>
#include <animal/img.h>
#include <animal/adjacency.h>

BEGIN_C_DECLS

// Dinstance Transform algorithm options
typedef enum {
   DT_EXACT_DILATIONS,   // Costa & Estrozi 1999 algorithm  (exact; slow)

   DT_IFT, DT_IFT_8,     // Falcao & Da Cunha algorithm (almost exact; fast)    
   DT_IFT_4,             // Same alg, w/4-neighborhood. You may call this alg. 
                         // with arbitrary neighborhood using its function directly.
   DT_LOTUFO_ZAMPIROLLI, // R. Lotufo & F. Zampirolli 1999 algorithm (exact; very fast)

   DT_MAURER2003,            // C. Maurer et. al. 2003 algorithm (exact, very fast)

   DT_CUISENAIRE_PMN_1999,   // O. Cuisenaire CVIU 1999 (exact, very fast)
   DT_CUISENAIRE_PMON_1999,  // O. Cuisenaire CVIU 1999 (exact, very fast)
   DT_CUISENAIRE_PSN4_1999,  // Inexact, very fast
   DT_CUISENAIRE_PSN8_1999,  // A little less inexact, very fast
} dt_algorithm;

AnimalExport ImgPUInt32
   *distance_transform(Img *bin, dt_algorithm alg);

AnimalExport bool
   edt_lz (ImgPUInt32 *im),
   edt_maurer2003(ImgPUInt32 *im),
   edt_ift(ImgPUInt32 *im, nhood *adj),
   edt_exact_dilations(ImgPUInt32 *im),
   edt_cuisenaire_psn4(ImgPUInt32 *im),  // simple 4-neighborhood approx. EDT
   edt_cuisenaire_psn8(ImgPUInt32 *im),  // simple 8-nhood approx. EDT
   edt_cuisenaire_pmn (ImgPUInt32 *im),  // multiple nhood, exact EDT
   edt_cuisenaire_pmon(ImgPUInt32 *im);  // multiple oriented nhood,

END_C_DECLS

#endif /* !DT_H */
